export declare type Role = string;
export interface UserAuthentication {
    token: string;
    username: string;
    realname: string;
    roles: Array<Role>;
}
export declare enum AuthenticationStatus {
    NONE = "NONE",
    CHECKING = "CHECKING",
    AUTHENTICATED = "AUTHENTICATED",
    UNAUTHENTICATED = "UNAUTHENTICATED",
    ERROR = "ERROR"
}
interface AuthenticationBase {
    status: AuthenticationStatus;
}
export interface AuthenticationNone extends AuthenticationBase {
    status: AuthenticationStatus.NONE;
}
export interface AuthenticationChecking extends AuthenticationBase {
    status: AuthenticationStatus.CHECKING;
}
export interface AuthenticationAuthenticated extends AuthenticationBase {
    status: AuthenticationStatus.AUTHENTICATED;
    userAuthentication: UserAuthentication;
}
export interface AuthenticationUnauthenticated extends AuthenticationBase {
    status: AuthenticationStatus.UNAUTHENTICATED;
}
export interface AuthenticationError extends AuthenticationBase {
    status: AuthenticationStatus.ERROR;
    message: string;
}
export declare type Authentication = AuthenticationNone | AuthenticationChecking | AuthenticationAuthenticated | AuthenticationUnauthenticated | AuthenticationError;
export default class Auth {
    url: string;
    constructor(url: string);
    checkAuth(): Promise<Authentication>;
}
export {};
