"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const PubSub_1 = __importStar(require("./PubSub"));
const TIMEOUT = 10000;
let spy;
beforeEach(() => {
    spy = jest.spyOn(console, 'error').mockImplementation(() => { });
});
afterEach(() => {
    if (spy) {
        spy.mockRestore();
    }
});
test('listen for and receive a simple message', () => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        pubsub.on('test', (message) => {
            expect(message).toEqual({ it: 'works' });
            resolve(null);
        });
        pubsub.send('test', { it: 'works' });
    });
}));
test('listen for and receive multiple simple messages', () => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        let messageReceivedCount = 0;
        pubsub.on('test', (message) => {
            expect(message).toEqual({ it: 'works' });
            messageReceivedCount += 1;
        });
        window.setTimeout(() => {
            expect(messageReceivedCount).toEqual(3);
            resolve(null);
        }, 1000);
        pubsub.send('test', { it: 'works' });
        pubsub.send('test', { it: 'works' });
        pubsub.send('test', { it: 'works' });
    });
}));
test('send non-existent message', () => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        let shouldNotBeSet = null;
        pubsub.on('test', (message) => {
            shouldNotBeSet = true;
        });
        pubsub.send('test2', { it: 'does nothing' });
        window.setTimeout(() => {
            expect(shouldNotBeSet).toBeNull();
            resolve(null);
        }, 1000);
    });
}));
test('receive an error which generates an error ', () => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        pubsub.on('test', (message) => {
            throw new Error('an error');
        }, (err) => {
            expect(err.message).toEqual('an error');
            resolve(null);
        });
        pubsub.send('test', { it: 'generates error!' });
    });
}));
test('receive an error which generates an error ', () => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        pubsub.on('test', (message) => {
            throw new Error('an error');
        });
        pubsub.send('test', { it: 'generates error!' });
        window.setTimeout(() => {
            expect(console.error).toHaveBeenCalled();
            expect(spy.mock.calls[0][0]).toContain('ERROR not handled: an error');
            resolve(null);
        }, 1000);
    });
}));
test('receive an error which generates an error in the error handler! ', () => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        pubsub.on('test', (message) => {
            throw new Error('an error');
        }, (err) => {
            throw new Error('another error!');
        });
        window.setTimeout(() => {
            expect(console.error).toHaveBeenCalled();
            expect(spy.mock.calls[0][0]).toEqual('ERROR in error handler: another error!');
            resolve(null);
        }, 1000);
        pubsub.send('test', { it: 'generates error!' });
    });
}));
test('send a message, receive, off it, send again, does nothing', () => __awaiter(void 0, void 0, void 0, function* () {
    const pubsub = new PubSub_1.default();
    let temp = null;
    let id = pubsub.on('test', (message) => {
        expect(message).toEqual({ it: 'works' });
        temp = 'works';
    });
    ;
    return new Promise((resolve) => {
        pubsub.send('test', { it: 'works' });
        window.setTimeout(() => {
            expect(temp).toEqual('works');
            resolve(null);
        }, 1000);
    })
        .then(() => {
        pubsub.off(id);
        temp = null;
        pubsub.send('test', { it: 'works' });
        return new Promise((resolve) => {
            window.setTimeout(() => {
                expect(temp).toBeNull();
                resolve(null);
            }, 1000);
        });
    });
}));
test('send a message, receive, wrong off it, send again, still works', () => __awaiter(void 0, void 0, void 0, function* () {
    const pubsub = new PubSub_1.default();
    let temp = null;
    pubsub.on('test', (message) => {
        expect(message).toEqual({ it: 'works' });
        temp = 'works';
    });
    ;
    return new Promise((resolve) => {
        pubsub.send('test', { it: 'works' });
        window.setTimeout(() => {
            expect(temp).toEqual('works');
            resolve(null);
        }, 1000);
    })
        .then(() => {
        pubsub.off('123');
        temp = null;
        pubsub.send('test', { it: 'works' });
        return new Promise((resolve) => {
            window.setTimeout(() => {
                expect(temp).toEqual('works');
                resolve(null);
            }, 1000);
        });
    });
}));
test('send and receive via the proxy', () => {
    return new Promise((resolve) => {
        const pubsub = new PubSub_1.default();
        const proxy = new PubSub_1.PubSubProxy(pubsub);
        proxy.on('test', (message) => {
            expect(message).toEqual({ it: 'works' });
            resolve(null);
        });
        pubsub.send('test', { it: 'works' });
    });
});
function doTimes(times, doit) {
    for (let i = 0; i < 10; i += 1) {
        doit(i);
    }
}
test('create and remove several listeners', () => __awaiter(void 0, void 0, void 0, function* () {
    const pubsub = new PubSub_1.default();
    const proxy = new PubSub_1.PubSubProxy(pubsub);
    let temp = new Array(10);
    doTimes(10, (i) => {
        temp[i] = null;
        proxy.on(`test${i}`, (message) => {
            expect(message).toEqual({ it: `works ${i}` });
            temp[i] = `works ${i}`;
        });
        ;
    });
    return new Promise((resolve) => {
        doTimes(10, (i) => {
            proxy.send(`test${i}`, { it: `works ${i}` });
            temp[i] = null;
        });
        window.setTimeout(() => {
            doTimes(10, (i) => {
                expect(temp[i]).toEqual(`works ${i}`);
            });
            resolve(null);
        }, 1000);
    })
        .then(() => {
        proxy.off();
        doTimes(10, (i) => {
            proxy.send(`test${i}`, { it: `works ${i}` });
            temp[i] = null;
        });
        return new Promise((resolve) => {
            window.setTimeout(() => {
                doTimes(10, (i) => {
                    expect(temp[i]).toBeNull();
                });
                resolve(null);
            }, 1000);
        });
    });
}));
//# sourceMappingURL=PubSub.test.js.map