export interface CacheParams {
    itemLifetime: number;
    monitoringFrequency: number;
    waiterTimeout: number;
    waiterFrequency: number;
}
export declare enum CacheItemState {
    RESERVED = 0,
    PRESENT = 1
}
export interface CacheItemBase<T> {
    state: CacheItemState;
    id: string;
    fetcher: () => Promise<T>;
}
export interface CacheItemReserved<T> extends CacheItemBase<T> {
    state: CacheItemState.RESERVED;
    reservedAt: number;
}
export interface CacheItemPresent<T> extends CacheItemBase<T> {
    state: CacheItemState.PRESENT;
    createdAt: number;
    value: T;
}
export declare type CacheItem<T> = CacheItemReserved<T> | CacheItemPresent<T>;
export declare type Fetcher<T> = () => Promise<T>;
export default class Cache<T> {
    cache: Map<string, CacheItem<T>>;
    cacheLifetime: number;
    monitoringFrequency: number;
    waiterTimeout: number;
    waiterFrequency: number;
    isMonitoring: boolean;
    constructor({ itemLifetime, monitoringFrequency, waiterTimeout, waiterFrequency }: CacheParams);
    private runMonitor;
    private isExpired;
    private getItem;
    private reserveWaiter;
    private reserveAndFetch;
    getItemWithWait({ id, fetcher }: {
        id: string;
        fetcher: Fetcher<T>;
    }): Promise<T>;
    private reserveItem;
}
