"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const windowChannel_1 = require("./windowChannel");
const simpleObject_1 = require("./simpleObject");
function isEqual(v1, v2) {
    const t1 = typeof v1;
    const t2 = typeof v2;
    if (t1 !== t2) {
        return false;
    }
    switch (t1) {
        case 'number':
        case 'string':
        case 'boolean':
        case 'undefined':
        case 'symbol':
            return v1 === v2;
        case 'object':
            if (Array.isArray(v1)) {
                if (Array.isArray(v2)) {
                    if (v1.length !== v2.length) {
                        return false;
                    }
                    for (const i in v1) {
                        if (!isEqual(v1[i], v2[i])) {
                            return false;
                        }
                    }
                    return true;
                }
                else {
                    return false;
                }
            }
            if (simpleObject_1.isSimpleObject(v1)) {
                if (simpleObject_1.isSimpleObject(v2)) {
                    const k1 = Object.keys(v1).sort();
                    const k2 = Object.keys(v2).sort();
                    if (k1.length !== k2.length) {
                        return false;
                    }
                    if (!isEqual(k1, k2)) {
                        return false;
                    }
                    for (const k of k1) {
                        if (!isEqual(v1[k], v2[k])) {
                            return false;
                        }
                    }
                    return true;
                }
                else {
                    return false;
                }
            }
            return false;
        default: return false;
    }
}
const WAIT_FOR_TIME = 5000;
const WAIT_FOR_INTERVAL = 50;
function myWaitFor(pred, timeout = WAIT_FOR_TIME) {
    const started = Date.now();
    return new Promise((resolve, reject) => {
        function loop() {
            if (pred()) {
                resolve();
                return;
            }
            const elapsed = Date.now() - started;
            if (elapsed >= timeout) {
                reject();
                return;
            }
            window.setTimeout(() => {
                loop();
            }, WAIT_FOR_INTERVAL);
        }
        loop();
    });
}
test('Window Channel create, send, receive simple message', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    expect(chan1.getId()).toEqual(chan1Init.getId());
    expect(chan2.getId()).toEqual(chan2Init.getId());
    expect(chan1.getPartnerId()).toEqual(chan2Init.getId());
    expect(chan2.getPartnerId()).toEqual(chan1Init.getId());
    expect(chan1.getStats()).toEqual({
        sent: 0, received: 0, ignored: 0
    });
    expect(chan2.getStats()).toEqual({
        sent: 0, received: 0, ignored: 0
    });
    let result;
    chan1.on('greeting', (message) => {
        result = message;
    });
    const payload = { hello: 'hi' };
    chan2.send('greeting', payload);
    yield myWaitFor(() => {
        return isEqual(result, payload);
    });
    expect(chan1.getStats()).toEqual({
        sent: 0, received: 1, ignored: 0
    });
    expect(chan2.getStats()).toEqual({
        sent: 1, received: 0, ignored: 1
    });
    chan1.stop();
    chan2.stop();
}));
test('Send a message as a promise', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    expect(chan1.getId()).toEqual(chan1Init.getId());
    expect(chan2.getId()).toEqual(chan2Init.getId());
    expect(chan1.getPartnerId()).toEqual(chan2Init.getId());
    expect(chan2.getPartnerId()).toEqual(chan1Init.getId());
    chan1.on('query', (message) => {
        return 'bar';
    });
    const payload = { name: 'foo' };
    const response = yield chan2.request('query', payload);
    expect(response).toEqual('bar');
    chan1.stop();
    chan2.stop();
}));
test('Window Channel create, send, receive SINGLE message', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    let result;
    chan1.once('greeting', (message) => {
        result = message;
    });
    const payload = { hello: 'hi' };
    chan2.send('greeting', payload);
    yield myWaitFor(() => {
        return isEqual(result, payload);
    });
    chan1.stop();
    chan2.stop();
}));
test('Window Channel create, send, receive SINGLE message', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    const payload = { hello: 'hi' };
    setTimeout(() => {
        chan2.send('greeting', payload);
    }, 0);
    const result = yield chan1.when('greeting', 1000);
    expect(result).toEqual(payload);
    chan1.stop();
    chan2.stop();
}));
test('Window Channel create, send, receive simple message', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId());
    expect(() => {
        chan1.send('greeting', { hello: 'hi' });
    }).toThrow(Error);
}));
test('Window Channel create, send bad message', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    window.postMessage(null, window.location.origin);
    window.postMessage({ hi: 'there' }, window.location.origin);
    window.postMessage(true, window.location.origin);
    window.postMessage(false, window.location.origin);
    window.postMessage(undefined, window.location.origin);
    window.postMessage(123, window.location.origin);
    window.postMessage([1, "2", 3.4], window.location.origin);
    yield new Promise((resolve) => {
        window.setTimeout(() => {
            resolve();
        }, 1000);
    });
    expect(chan1.getStats()).toEqual({
        sent: 0, received: 0, ignored: 7
    });
    expect(chan2.getStats()).toEqual({
        sent: 0, received: 0, ignored: 7
    });
    chan1.stop();
    chan2.stop();
}));
test('Send a request, handler generates exception', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    expect(chan1.getId()).toEqual(chan1Init.getId());
    expect(chan2.getId()).toEqual(chan2Init.getId());
    expect(chan1.getPartnerId()).toEqual(chan2Init.getId());
    expect(chan2.getPartnerId()).toEqual(chan1Init.getId());
    chan1.on('query', (message) => {
        throw new Error('Oops!');
    });
    const payload = { name: 'foo' };
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () {
        try {
            return yield chan2.request('query', payload);
        }
        catch (ex) {
            throw ex;
        }
    })).rejects.toThrowError('Oops!');
    chan1.stop();
    chan2.stop();
}));
test('Window Channel create, send, receive message with error', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    let error = null;
    chan1.on('greeting', (message) => {
        throw new Error('Whoopsie!');
    }, (err) => {
        error = err;
    });
    const payload = { hello: 'hi' };
    chan2.send('greeting', payload);
    yield myWaitFor(() => {
        if (error != null) {
            return error.message === 'Whoopsie!';
        }
        return false;
    });
    chan1.stop();
    chan2.stop();
}));
test('Window Channel create, send, receive SINGLE message with error', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId()).start();
    const chan2 = chan2Init.makeChannel(chan1Init.getId()).start();
    let error = null;
    chan1.once('greeting', (message) => {
        throw new Error('Whoopsie!');
    }, (err) => {
        error = err;
    });
    const payload = { hello: 'hi' };
    chan2.send('greeting', payload);
    yield myWaitFor(() => {
        if (error != null) {
            return error.message === 'Whoopsie!';
        }
        return false;
    });
    chan1.stop();
    chan2.stop();
}));
test('Cannot send unless start first', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId());
    const chan2 = chan2Init.makeChannel(chan1Init.getId());
    const payload = { hello: 'hi' };
    expect(() => {
        chan2.send('greeting', payload);
    }).toThrow();
}));
test('Cannot request unless start first', () => __awaiter(void 0, void 0, void 0, function* () {
    const chan1Init = new windowChannel_1.WindowChannelInit();
    const chan2Init = new windowChannel_1.WindowChannelInit();
    const chan1 = chan1Init.makeChannel(chan2Init.getId());
    const chan2 = chan2Init.makeChannel(chan1Init.getId());
    const payload = { hello: 'hi' };
    yield expect(chan2.request('greeting', payload)).rejects.toThrow();
}));
//# sourceMappingURL=windowChannel.test.js.map